/**************************************************************************************************
  Filename:       simpleBLEperipheral.h
  Revised:        $Date: 2010-08-01 14:03:16 -0700 (Sun, 01 Aug 2010) $
  Revision:       $Revision: 23256 $

  Description:    This file contains the Simple BLE Peripheral sample application
                  definitions and prototypes.

  Copyright 2010 - 2011 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
**************************************************************************************************/

#ifndef MODBUSPERIPHERAL_H
#define MODBUSPERIPHERAL_H

#ifdef __cplusplus
extern "C"
{
#endif

/*********************************************************************
 * INCLUDES
 */

/*********************************************************************
 * CONSTANTS
 */


// Simple BLE Peripheral Task Events
#define MBP_START_DEVICE_EVT                              0x0001
#define MBP_PERIODIC_EVT                                  0x0400
#define MBP_KEY_EVT                                       0x1000
#define MBP_ADV_OFF_EVT                                   0x0008
#define MBP_LED_BLINK_EVT                                 0x0010
#define MBP_FRAGMENT_TIMEOUT_EVT                          0x0020
#define MBP_FRAGMENT_READY_EVT                            0x0040
#define MBP_FINAL_FRAGMENT_EVT                            0x0080
   
//#define LED_OFF_EVT                                        0x0100        

  
/*********************************************************************
 * MACROS
 */
   //LED macros
   
#if defined CC2540_RS485
  #define LED_BLINK_START() osal_stop_timerEx(modbusTaskId, MBP_LED_BLINK_EVT); \
                            osal_set_event(modbusTaskId, MBP_LED_BLINK_EVT)
  #define LED_BLINK_STOP()  osal_stop_timerEx(modbusTaskId, MBP_LED_BLINK_EVT)
  #define LED_ON()          P1 |= 0x02                                  
  #define LED_OFF()         P1 &= ~0x02
  #define LED_TOGGLE()      if(P1 & 0x02) LED_OFF(); else LED_ON()                                         
#else                                         
  #define LED_BLINK_START()                                         
  #define LED_BLINK_STOP()                                         
  #define LED_ON()                                         
  #define LED_OFF()                                         
  #define LED_TOGGLE()                                         
#endif
   
#define SET_TEST()      P0 |= 0x40;
#define CLEAR_TEST()    P0 &= ~0x40;


/*********************************************************************
 * FUNCTIONS
 */

/*
 * Task Initialization for the BLE Application
 */
extern void ModbusPeripheral_Init( uint8 task_id );

/*
 * Task Event Processor for the BLE Application
 */
extern uint16 ModbusPeripheral_ProcessEvent( uint8 task_id, uint16 events );

/*********************************************************************
*********************************************************************/

#ifdef __cplusplus
}
#endif

#endif /* MODBUSPERIPHERAL_H */
